import { isEmpty } from 'lodash'
import type { IASMAAppsObject } from './interfaces'

import { registerMicroApps } from './qiankun/apis'
/**
 *@readonly do not update directly use setAppsObject() in stead
 */
let appsObject: IASMAAppsObject | undefined

export function getASMAAppsObject() {
    return appsObject
}

export function setAppsObject(appsObject: IASMAAppsObject) {
    appsObject = appsObject
}

export async function registerASMAMicroApps(
    render: ({ loading }: { loading: boolean }) => void,
    apps_object: IASMAAppsObject = {},
    devtools = false,
) {
    const loader = (loading: boolean) => render({ loading })

    const apps_arr = Object.values(apps_object).map((mf) => ({ ...mf, loader })) ?? []

    if (devtools) {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        const _ = await import('qiankun-overrides')

        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        const imo: any = window.importMapOverrides

        imo?.setDefaultMap(apps_arr)

        appsObject = imo?.getCurrentQiankunMap()
    }

    if (appsObject && !isEmpty(appsObject)) {
        registerMicroApps(
            Object.values(appsObject).filter((app) => app.activeRule !== 'component-only'),
            {
                beforeLoad: async (app) => {
                    console.log('[LifeCycle] before load', 'color: green;', app.name)
                },

                beforeMount: async (app) => {
                    console.log('[LifeCycle] before mount', 'color: green;', app.name)
                },

                afterUnmount: async (app) => {
                    console.log('[LifeCycle] after unmount', 'color: green;', app.name)
                },
            },
        )
    }
}
